#region Using

using System;
using System.Web;
using System.Collections.Generic;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using BlogEngine.Core;
using System.Web.Caching;

#endregion

public partial class archivesummary : BlogEngine.Core.Web.Controls.BlogBasePage
{
    private const double CacheTimeoutInHours = 1;

    static archivesummary()
    {
        Post.Saved += Post_Saved;
    }

    static void Post_Saved(object sender, SavedEventArgs e)
    {
        // invalidate cache whenever a post is modified
        lock (_SyncRoot)
        {
            HttpRuntime.Cache.Remove(CacheKey);
        }

    }

    #region Private fields

    private readonly static object _SyncRoot = new object();
    private const string CacheKey = "BE_ArchiveMonthListCacheKey";
    #endregion

    /// <summary>
    /// Handles the Load event of the Page control.
    /// </summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
    protected void Page_Load(object sender, EventArgs e)
    {
        if (!IsPostBack && !IsCallback)
        {
            HtmlGenericControl style = new HtmlGenericControl("style");
            //style.Attributes.Add("type", "text/css");
            style.InnerHtml = @".linkover {margin:0 0 5px 0;padding:10px 0;clear:both;}
                                .linkover li {width:128px;line-height:20px;margin:0 10px;float:left;}
                                .linkover li a {font-weight:bold;}";
            Header.Controls.Add(style);
            CreateMonth();
            CreateCategories();
            AddTotals();
        }

        Page.Title = Server.HtmlEncode(Resources.labels.archive);
        base.AddMetaTag("description", Resources.labels.archive + " | " + BlogSettings.Instance.Name);
    }

    private void CreateMonth()
    {
        HtmlGenericControl h2 = new HtmlGenericControl("h2");
        h2.InnerText = "·";
        phArchive.Controls.Add(h2);

        SortedDictionary<DateTime, int> months = GetPostsPerMonth();
        if (months.Keys.Count == 0)
        {
            HtmlGenericControl p = new HtmlGenericControl("p");
            p.InnerText = Resources.labels.none;
            phArchive.Controls.Add(p);
            return;
        }

        HtmlGenericControl ul = new HtmlGenericControl("ul");
        ul.Attributes.Add("class", "linkover");

        //int currentYear = 0;

        foreach (DateTime date in months.Keys)
        {
             //if (currentYear == 0)
             //    currentYear = date.Year;

             //if (date.Year > currentYear || ul.Controls.Count == 0)
             //{
             //    if (ul.Controls.Count > 0)
             //    {
             //        HtmlGenericControl br = new HtmlGenericControl("br");
             //        ul.Controls.Add(br);
             //    }

             //    HtmlGenericControl p = new HtmlGenericControl("p");
             //    p.Attributes.Add("class", "linkgroup");
             //    p.InnerHtml = date.Year.ToString();
             //    ul.Controls.Add(p);
             //}

            HtmlGenericControl li = new HtmlGenericControl("li");

            HtmlAnchor anc = new HtmlAnchor();
            anc.HRef = Utils.RelativeWebRoot + date.Year + "/" + date.ToString("MM") + "/default" + BlogSettings.Instance.FileExtension;
            anc.InnerHtml = DateTime.Parse(date.Year + "-" + date.Month + "-01").ToString("yyyy  MMMM") + " (" + months[date] + ")";

            li.Controls.Add(anc);
            ul.Controls.AddAt(0, li);
            //currentYear = date.Year;
        }
        
        phArchive.Controls.Add(ul);

        HtmlGenericControl div = new HtmlGenericControl("div");
        div.Attributes.Add("style", "clear:both");
        phArchive.Controls.Add(div);
    }

    private static SortedDictionary<DateTime, int> GetPostsPerMonth()
    {
        lock (_SyncRoot)
        {
            SortedDictionary<DateTime, int> months = HttpRuntime.Cache[CacheKey] as SortedDictionary<DateTime, int>;
            if (months == null)
            {
                months = new SortedDictionary<DateTime, int>();
                // let dictionary expire after 1 hour
                HttpRuntime.Cache.Insert(CacheKey, months, null, DateTime.Now.AddHours(CacheTimeoutInHours), Cache.NoSlidingExpiration);

                foreach (Post post in Post.Posts)
                {
                    if (post.IsVisibleToPublic)
                    {
                        DateTime month = new DateTime(post.DateCreated.Year, post.DateCreated.Month, 1);
                        int count;
                        // if the date is not in the dictionary, count will be set to 0
                        months.TryGetValue(month, out count);
                        ++count;
                        months[month] = count;
                    }
                }
            }
            return months;
        }
    }
   
    private void CreateCategories()
    {
        HtmlGenericControl h2 = new HtmlGenericControl("h2");
        h2.InnerText = "";
        phArchive.Controls.Add(h2);

        if (Category.Categories.Count == 0)
        {
            HtmlGenericControl p = new HtmlGenericControl("p");
            p.InnerText = Resources.labels.none;
            phArchive.Controls.Add(p);
            return;
        }

        HtmlGenericControl ul = new HtmlGenericControl("ul");
        ul.Attributes.Add("class", "linkover");

        foreach(Category cat in Category.Categories)
        {
            int posts = 0;
            try
            {
                posts = Post.GetPostsByCategory(cat.Id).FindAll(delegate(Post p)
                {
                    return p.IsVisible;
                }).Count;
            }
            catch { }

            HtmlGenericControl li = new HtmlGenericControl("li");

            HtmlImage img = new HtmlImage();
            img.Src = Utils.RelativeWebRoot + "pics/rssButton.gif";
            img.Alt = BlogSettings.Instance.SyndicationFormat.ToUpperInvariant() + " feed for " + cat.CompleteTitle();
            img.Attributes["class"] = "rssButton";
            img.Style.Add(HtmlTextWriterStyle.MarginRight, "2px");
            
            HtmlAnchor feedAnchor = new HtmlAnchor();
            feedAnchor.HRef = cat.FeedRelativeLink;
            feedAnchor.Attributes["rel"] = "nofollow";
            feedAnchor.Controls.Add(img);

            li.Controls.Add(feedAnchor);

            HtmlAnchor anc = new HtmlAnchor();
            anc.HRef = cat.RelativeLink;
            anc.InnerHtml = HttpUtility.HtmlEncode(cat.CompleteTitle()) + " (" + posts + ")";
            anc.Title = "Category: " + cat.CompleteTitle();

            li.Controls.Add(anc);
            ul.Controls.Add(li);
        }

        phArchive.Controls.Add(ul);

        HtmlGenericControl div = new HtmlGenericControl("div");
        div.Attributes.Add("style", "clear:both");
        phArchive.Controls.Add(div);
    }

    private void AddTotals()
    {
        int comments = 0;
        int raters = 0;
        List<Post> posts = Post.Posts.FindAll(delegate(Post p) { return p.IsVisible; });
        foreach (Post post in posts)
        {
            comments += post.ApprovedComments.Count;
            raters += post.Raters;
        }

        ltPosts.Text = posts.Count + " " + Resources.labels.posts.ToLowerInvariant();
        if (BlogSettings.Instance.IsCommentsEnabled)
            ltComments.Text = comments + " " + Resources.labels.comments.ToLowerInvariant();

        if (BlogSettings.Instance.EnableRating)
            ltRaters.Text = raters + " " + Resources.labels.raters.ToLowerInvariant();
    }
}
